//
//  Created by ITCowork Inc on 2014/02/18.
//  Copyright (c) 2014 ITCowork. All rights reserved.
//

#import "IOSAnnotationView.h"
#import "IOSAnnotation.h"
#import "InfoWindow.h"
#import "NSObject+Extension.h"

@interface IOSAnnotationView()
@property (nonatomic, assign) BOOL isLayouted;
@property (strong, nonatomic) InfoWindow *infoWindow;
@end

@implementation IOSAnnotationView

- (id)initWithAnnotation:(id<MKAnnotation>)annotation reuseIdentifier:(NSString *)reuseIdentifier
{
    self = [super initWithAnnotation:annotation reuseIdentifier:reuseIdentifier];
    if(self)
    {
        _isLayouted = NO;
        _tmpCalloutView = nil;
        _infoWindow = nil;
        self.backgroundColor = [UIColor clearColor];
    }
    return self;
}

- (void)layoutSubviews
{
    @synchronized(self)
    {
        @autoreleasepool
        {
            if(self.isLayouted == NO)
            {
                self.isLayouted = YES;
                
                //オリジナルコールアウト
                self.infoWindow = [InfoWindow instanceWithAnnotation:self.annotation];
                
                CGRect pinRect = [self convertRect:self.bounds toView:self.tmpCalloutView];
                CGFloat px = CGRectGetMidX(pinRect);
                CGFloat py = CGRectGetMinY(pinRect) - [self division:CGRectGetHeight(self.infoWindow.view.bounds) by:2.0];
                
                //ピンの画像によっては微調整
                px = px - 8.0f;
                py = py - 3.0f;
                
                [self.infoWindow.view setCenter:CGPointMake(px, py)];
                
                [self asyncAfterDelay:0.0f block:^{
                    
                    //iOS7系の場合はアニメーション
                    if(floor(NSFoundationVersionNumber) > NSFoundationVersionNumber_iOS_6_1)
                    {
                        [self set_popup_animation:self.infoWindow.view];
                    }
                    
                    [self.tmpCalloutView addSubview:self.infoWindow.view];
                }];
            }
        }
    }
}

- (void)didAddSubview:(UIView *)subview
{
    //コールアウトへの参照を保持
    //コールアウト上のSubViewをクリア
    if([NSStringFromClass([subview class]) isEqualToString:@"UICalloutView"] ||
       [NSStringFromClass([subview class]) isEqualToString:@"UIView"])
    {
        self.tmpCalloutView = subview;
        for(UIView *sv in [self.tmpCalloutView subviews])
        {
            [sv removeFromSuperview];
        }
        self.tmpCalloutView.backgroundColor = [UIColor clearColor];
        self.tmpCalloutView.clipsToBounds = NO;
        self.isLayouted = NO;
    }
}

- (CGRect)infoWindowFrame
{
    return [[self superview] convertRect:self.infoWindow.view.frame fromView:self.tmpCalloutView];;
}

@end
